#!/usr/bin/env python3
# coding: utf-8

"""
    %prog [options] [repository_path]

Show the last current tag and compute the next one
"""

import os.path as osp
import re
import time
from functools import partial
from optparse import OptionParser
from subprocess import PIPE, Popen

from aslint.logger import logger, setlevel
from aslint.string_utils import convert

FMTTAG = "([0-9]+)\.([0-9]+)\.([0-9]+)"


def next_tag(repo, branch=".", offset=0, verbose=False):
    """Return the tag of the next minor version"""
    cmd = [
        "hg",
        "log",
        "-R",
        repo,
        "--rev",
        "last(tag() and branch(%s))" % branch,
        "--template",
        "{tags}\n",
    ]
    logger.debug("Command: {0}".format(cmd))
    tag = convert(Popen(cmd, stdout=PIPE).communicate()[0]).strip()
    if not tag:
        raise ValueError(
            "No tag found in this branch. This may happen on a "
            "newly created branch. Define OLD and TAG yourself!"
        )
    mat = re.search(FMTTAG, tag)
    if not mat:
        raise ValueError("invalid tag: %r" % tag)
    tag = re.findall(FMTTAG, tag)
    branch, major, minor = tag[0]
    old = "%s.%s.%s" % (branch, major, int(minor) + offset)
    ntag = "%s.%s.%s" % (branch, major, int(minor) + offset + 1)
    if verbose:
        logger.info("OLD=%s", old)
        logger.info("TAG=%s", ntag)
    return ntag


if __name__ == "__main__":
    # command arguments parser
    parser = OptionParser(usage=__doc__)
    parser.add_option(
        "-g", "--debug", action="callback", callback=setlevel, help="add debug informations"
    )
    parser.add_option(
        "--offset",
        action="store",
        default=0,
        type=int,
        help="offset patch level (-1 to get the previous tags)",
    )
    parser.add_option(
        "--branch", action="store", default=".", help="branch to be used to search tags."
    )
    opts, args = parser.parse_args()
    repo = "."
    if len(args) > 0:
        repo = args[0]
    if len(args) > 1:
        parser.error("at most one argument is required")
    next_tag(repo, opts.branch, opts.offset, verbose=True)
